@@ -5,9 +5,9 @@ $|++ ;
 
 use strict;
 use warnings;
+use utf8;
 use Carp ;
 
-use Data::Dumper ;
 use Data::TreeDumper ;
 use File::Slurp ;
 use Clone;
@@ -15,76 +15,15 @@ use List::Util qw(min max) ;
 use List::MoreUtils qw(any minmax first_value) ;
 use Readonly ;
 
-use Glib ':constants';
-use Gtk2 -init;
-use Gtk2::Gdk::Keysyms ;
-my %K = %Gtk2::Gdk::Keysyms ;
-
 use App::Asciio::Connections ;
 
 #-----------------------------------------------------------------------------
 
-sub set_modified_state
-{
-my ($self, $state) = @_ ;
-$self->{MODIFIED} = $state ;
-}
+sub set_modified_state { my ($self, $state) = @_ ; $self->{MODIFIED} = $state ; }
 
 #-----------------------------------------------------------------------------
 
-sub get_modified_state
-{
-my ($self) = @_ ;
-$self->{MODIFIED} ;
-}
-
-#-----------------------------------------------------------------------------
-
-sub get_color 
-{
-my ($self, $name) = @_;
-
-unless (exists $self->{ALLOCATED_COLORS}{$name}) 
-	{
-	my $color  ;
-	
-	if('ARRAY' eq  ref $name) 
-		{
-		$color = Gtk2::Gdk::Color->new( map {$_ * 257} @{$name}) ;
-		}
-	elsif(exists $self->{COLORS}{$name}) 
-		{
-		if('ARRAY' eq ref $self->{COLORS}{$name})
-			{
-			$color = Gtk2::Gdk::Color->new( map {$_ * 257} @{ $self->{COLORS}{$name}}) ;
-			}
-		else
-			{
-			$color = Gtk2::Gdk::Color->parse($self->{COLORS}{$name});
-			}
-		}
-	else
-		{
-		$color = Gtk2::Gdk::Color->parse($name);
-		}
-	
-	$color = Gtk2::Gdk::Color->new( map {$_ * 257} (255, 0, 0)) unless defined $color ;
-	$self->{widget}->get_colormap->alloc_color($color,TRUE,TRUE) ;
-	
-	$self->{ALLOCATED_COLORS}{$name} = $color ;
-	}
-	
-return($self->{ALLOCATED_COLORS}{$name}) ;
-}
-
-#-----------------------------------------------------------------------------
-
-sub flush_color_cache
-{
-my ($self) = @_ ;
-delete $self->{ALLOCATED_COLORS} ;
-}
-
+sub get_modified_state { my ($self) = @_ ; $self->{MODIFIED} ; }
 
 #-----------------------------------------------------------------------------
 
@@ -93,12 +32,12 @@ sub get_group_color
 # cycle through color to give visual clue to user
 my ($self) = @_ ;
 
-my $name = $self->{GROUP_COLORS}[$self->{NEXT_GROUP_COLOR}] ;
+my $colors = $self->{COLORS}{group_colors}[$self->{NEXT_GROUP_COLOR}] ;
 
 $self->{NEXT_GROUP_COLOR}++ ;
-$self->{NEXT_GROUP_COLOR} = 0 if $self->{NEXT_GROUP_COLOR} >= scalar(@{$self->{GROUP_COLORS}}) ;
+$self->{NEXT_GROUP_COLOR} = 0 if $self->{NEXT_GROUP_COLOR} >= scalar(@{$self->{COLORS}{group_colors}}) ;
 
-return ($name) ;
+return ($colors) ;
 }
 
 #-----------------------------------------------------------------------------
@@ -131,8 +70,8 @@ for my $ruler_line_to_remove (@ruler_lines_to_remove)
 			}
 		}
 	}
-	
-$self->{RULER_LINES} = [grep {! exists $removed{$_}} @{$self->{RULER_LINES}} ] ;
+
+$self->{RULER_LINES} = [ grep { ! exists $removed{$_} } @{$self->{RULER_LINES}} ] ;
 }
 
 sub exists_ruler_line
@@ -156,7 +95,7 @@ for my $ruler_line_to_check (@ruler_lines_to_check)
 			}
 		}
 	}
-	
+
 return $exists ;
 }
 
@@ -203,20 +142,33 @@ my ($self, $element, $x, $y) = @_ ;
 
 #-----------------------------------------------------------------------------
 
+sub set_elements_position
+{
+my ($self, $elements, $x, $y) = @_ ;
+
+@$_{'X', 'Y'} = ($x, $y) for @$elements ;
+}
+
+#-----------------------------------------------------------------------------
+
 sub add_element_at
 {
 my ($self, $element, $x, $y) = @_ ;
 
-$self->add_element_at_no_connection($element,$x, $y) ;
+$self->add_element_at_no_connection($element, $x, $y) ;
 $self->connect_elements($element) ;
+
+$element
 }
 
 sub add_element_at_no_connection
 {
 my ($self, $element, $x, $y) = @_ ;
 
-$self->set_element_position($element,$x, $y) ;
+$self->set_element_position($element, $x, $y) ;
 $self->add_elements_no_connection($element) ;
+
+$element
 }
 
 #-----------------------------------------------------------------------------
@@ -256,8 +208,8 @@ my %selected_elements = map { $_ => 1} @elements ;
 
 for my $element (@elements)
 	{
-	@$element{'X', 'Y'} =	($element->{X} + $x_offset, $element->{Y} + $y_offset) ;
-		
+	@$element{'X', 'Y'} = ($element->{X} + $x_offset, $element->{Y} + $y_offset) ;
+	
 	# handle arrow element
 	my (@current_element_connections, %used_connectors) ;
 	
@@ -266,7 +218,7 @@ for my $element (@elements)
 		# disconnect current connections if it is not connected to another elements we are moving
 		# connectees move their connected along
 		
-		@current_element_connections =$self->get_connections_containing($element)  ,
+		@current_element_connections = $self->get_connections_containing($element)  ,
 		
 		my (@connections_to_delete, @connections_to_keep) ;
 		for my $current_element_connection (@current_element_connections)
@@ -340,12 +292,12 @@ for my $element (@elements)
 				}
 			}
 		}
-		
+	
 	for my $connection (@{$self->{CONNECTIONS}})
 		{
 		delete $connection->{FIXED} ;
 		}
-			
+	
 	$self->{MODIFIED }++ ;
 	}
 }
@@ -354,7 +306,7 @@ for my $element (@elements)
 
 sub resize_element
 {
-my ($self, $reference_x, $reference_y, $new_x, $new_y, $selected_element, $connector_name) = @ _;
+my ($self, $reference_x, $reference_y, $new_x, $new_y, $selected_element, $connector_name) = @_;
 
 my ($x_offset, $y_offset, undef, undef, $resized_connector_name) = 
 	$selected_element->resize($reference_x, $reference_y, $new_x, $new_y, undef, $connector_name) ;
@@ -368,7 +320,7 @@ if($self->is_connected($selected_element))
 	# disconnect current connections
 	$self->delete_connections_containing($selected_element) ;
 	}
-	
+
 $self->connect_elements($selected_element) ; # connect to new elements if any
 
 for my $connection ($self->get_connected($selected_element))
@@ -376,7 +328,7 @@ for my $connection ($self->get_connected($selected_element))
 	# all connection where the selected element is the connectee
 	
 	my ($new_connection) = # in characters relative to element origin
-			$selected_element->get_named_connection($connection->{CONNECTION}{NAME}) ;
+	$selected_element->get_named_connection($connection->{CONNECTION}{NAME}) ;
 	
 	if(defined $new_connection)
 		{
@@ -421,7 +373,7 @@ for my $connection ($self->get_connected($selected_element))
 		$self->delete_connections($connection) ;
 		}
 	}
-	
+
 return($x_offset, $y_offset, $resized_connector_name) ;
 }
 
@@ -465,7 +417,7 @@ sub delete_elements
 {
 my($self, @elements) = @_ ;
 
-my %elements_to_delete = map {$_, 1}  @elements ;
+my %elements_to_delete = map {$_, 1} grep { defined $_ } @elements ;
 
 for my $element (@{$self->{ELEMENTS}})
 	{
@@ -475,7 +427,7 @@ for my $element (@{$self->{ELEMENTS}})
 		$element = undef ;
 		}
 	}
-	
+
 @{$self->{ELEMENTS}} = grep { defined $_} @{$self->{ELEMENTS}} ;
 
 $self->{MODIFIED }++ ;
@@ -487,7 +439,7 @@ sub edit_element
 {
 my ($self, $selected_element) = @_ ;
 
-$selected_element->edit() ;
+$selected_element->edit($self) ;
 
 # handle connections
 if($self->is_connected($selected_element))
@@ -495,10 +447,8 @@ if($self->is_connected($selected_element))
 	# disconnect current connections
 	$self->delete_connections_containing($selected_element) ;
 	}
-	
-#~ !!! TODO if not already connected to them (same connection)
-$self->connect_elements($selected_element) ; # connect to new elements if any
 
+$self->connect_elements($selected_element) ; # connect to new elements if any
 
 for my $connection ($self->get_connected($selected_element))
 	{
@@ -549,10 +499,8 @@ for my $connection ($self->get_connected($selected_element))
 		{
 		$self->delete_connections($connection) ;
 		}
-		
-	#~ TODO fix the other connection as move does above
 	}
-	
+
 $self->{MODIFIED }++ ;
 }
 
@@ -611,7 +559,7 @@ for my $element (@elements)
 		$groups_to_select{$element->{GROUP}[-1]}++ ;
 		}
 	}
-	
+
 # select groups
 for my $element (@{$self->{ELEMENTS}}) 
 	{
@@ -631,19 +579,65 @@ for my $element (@{$self->{ELEMENTS}})
 			}
 		}
 	}
-	
+
 delete $self->{SELECTION_INDEX} unless $self->get_selected_elements(1) ;
 }
 
 #-----------------------------------------------------------------------------
 
+sub select_all_elements_by_search_words
+{
+my ($self) = @_ ;
+
+my $search_words = $self->display_edit_dialog("input search words", '', $self);
+
+for my $element (@{$self->{ELEMENTS}}) 
+	{
+	$self->select_elements(1, $element) if ($self->transform_elements_to_ascii_buffer($element) =~ m/$search_words/i);
+	}
+}
+
+#-----------------------------------------------------------------------------
+
+sub select_all_elements_by_search_words_ignore_group
+{
+my ($self) = @_ ;
+
+my $search_words = $self->display_edit_dialog("input search words", '', $self);
+
+for my $element (@{$self->{ELEMENTS}}) 
+	{
+	$element->{SELECTED} = ++$self->{SELECTION_INDEX} if ($self->transform_elements_to_ascii_buffer($element) =~ m/$search_words/i);
+	}
+}
+
+#-----------------------------------------------------------------------------
+
+sub select_all_elements
+{
+my ($self) = @_ ;
+
+$self->select_elements(1, @{$self->{ELEMENTS}}) ;
+}
+
+#-----------------------------------------------------------------------------
+
+sub deselect_all_elements
+{
+my ($self) = @_ ;
+
+$self->select_elements(0, @{$self->{ELEMENTS}}) ;
+}
+
+#-----------------------------------------------------------------------------
+
 sub select_elements_flip
 {
 my ($self, @elements) = @_ ;
 
 for my $element (@elements) 
 	{
-	$self->select_elements($element->{SELECTED} ^ 1, $element)  ;
+	$self->select_elements($element->{SELECTED} ? 0 : 1, $element)  ;
 	}
 
 delete $self->{SELECTION_INDEX} unless $self->get_selected_elements(1) ;
@@ -664,27 +658,25 @@ sub is_over_element
 {
 my ($self, $element, $x, $y, $field) = @_ ;
 
-die "Error: 'is_over_element' needs position!" unless defined $x && defined $y ;
-
-$field ||= 0 ;
+$field //= 0 ;
 my $is_under = 0 ;
 
-for my $mask_strip ($element->get_mask_and_element_stripes())
+for my $strip (@{$element->get_stripes()})
 	{
-	my $stripe_x = $element->{X} + $mask_strip->{X_OFFSET} ;
-	my $stripe_y = $element->{Y} + $mask_strip->{Y_OFFSET} ;
+	my $stripe_x = $element->{X} + $strip->{X_OFFSET} ;
+	my $stripe_y = $element->{Y} + $strip->{Y_OFFSET} ;
 	
 	if
 		(
-		$stripe_x - $field <= $x   && $x < $stripe_x + $mask_strip->{WIDTH} + $field
-		&& $stripe_y - $field <= $y && $y < $stripe_y + $mask_strip->{HEIGHT} + $field
+		$stripe_x - $field <= $x   && $x < $stripe_x + $strip->{WIDTH} + $field
+		&& $stripe_y - $field <= $y && $y < $stripe_y + $strip->{HEIGHT} + $field
 		) 
 		{
 		$is_under++ ;
 		last ;
 		}
 	}
-	
+
 return($is_under) ;
 }
 
@@ -709,20 +701,20 @@ if($height < 0)
 	$height *= -1 ;
 	$start_y -= $height ;
 	}
-	
+
 my $is_under = 1 ;
 
-for my $mask_strip ($element->get_mask_and_element_stripes())
+for my $strip (@{$element->get_stripes()})
 	{
-	my $stripe_x = $element->{X} + $mask_strip->{X_OFFSET} ;
-	my $stripe_y = $element->{Y} + $mask_strip->{Y_OFFSET} ;
+	my $stripe_x = $element->{X} + $strip->{X_OFFSET} ;
+	my $stripe_y = $element->{Y} + $strip->{Y_OFFSET} ;
 	
 	if
 	(
 	    $start_x <= $stripe_x
-	&& ($stripe_x + $mask_strip->{WIDTH})  <= $start_x +$width
+	&& ($stripe_x + $strip->{WIDTH})  <= $start_x +$width
 	&& $start_y <= $stripe_y 
-	&& ($stripe_y + $mask_strip->{HEIGHT}) <= $start_y + $height
+	&& ($stripe_y + $strip->{HEIGHT}) <= $start_y + $height
 	) 
 		{
 		}
@@ -732,12 +724,33 @@ for my $mask_strip ($element->get_mask_and_element_stripes())
 		last
 		}
 	}
-	
+
 return($is_under) ;
 }
 
 #-----------------------------------------------------------------------------
 
+sub get_extent_box
+{
+my ($self, @elements) = @_ ;
+
+@elements = $self->get_selected_elements(1) unless @elements ;
+
+my ($xs, $ys, $xe, $ye) = (10_000, 10_000, 0, 0) ;
+
+for (grep { ref($_) !~ /arrow/ } @elements)
+	{
+	$xs = min($xs//10_000, $_->{X} + $_->{EXTENTS}[0]) ;
+	$ys = min($ys//10_000, $_->{Y} + $_->{EXTENTS}[1]) ;
+	$xe = max($xe//0, $_->{X} + $_->{EXTENTS}[2]) ;
+	$ye = max($ye//0, $_->{Y} + $_->{EXTENTS}[3]) ;
+	}
+
+($xs // 0, $ys // 0, $xe // 0, $ye // 0) ;
+}
+
+#-----------------------------------------------------------------------------
+
 sub pixel_to_character_x
 {
 my ($self, @pixels) = @_ ;
@@ -747,6 +760,8 @@ my ($character_width, $character_height) = $self->get_character_size() ;
 map {int($_ / $character_width)} @pixels ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub pixel_to_character_y
 {
 my ($self, @pixels) = @_ ;
@@ -772,28 +787,5 @@ return($character_x, $character_y) ;
 
 #-----------------------------------------------------------------------------
 
-sub get_character_size
-{
-my ($self) = @_ ;
-	
-if(exists $self->{USER_CHARACTER_WIDTH})
-	{
-	return ($self->{USER_CHARACTER_WIDTH}, $self->{USER_CHARACTER_HEIGHT}) ;
-	}
-else
-	{
-	my $layout = $self->{widget}->create_pango_layout('M') ;
-	return $layout->get_pixel_size() ;
-	}
-}
+1 ;
 
-sub set_character_size
-{
-my ($self, $width, $height) = @_ ;
-	
-($self->{USER_CHARACTER_WIDTH}, $self->{USER_CHARACTER_HEIGHT}) = ($width, $height) ;
-}
-
-#-----------------------------------------------------------------------------
-
-1 ;
\ No newline at end of file
