@@ -6,13 +6,21 @@ $|++ ;
 use strict;
 use warnings;
 
-use Data::Dumper ;
 use Data::TreeDumper ;
 use File::Slurp ;
 use Readonly ;
-#~ use Compress::LZF ':compress';
 use Compress::Bzip2 qw(:all :utilities :gzip);
 
+use Sereal qw(
+    get_sereal_decoder
+    get_sereal_encoder
+    clear_sereal_object_cache
+ 
+    encode_sereal
+    decode_sereal
+) ;
+use Sereal::Encoder qw(SRL_SNAPPY SRL_ZLIB SRL_ZSTD) ;
+
 #-----------------------------------------------------------------------------
 
 sub load_file
@@ -24,9 +32,7 @@ return unless defined $file_name ;
 my ($base_name, $path, $extension) = File::Basename::fileparse($file_name, ('\..*')) ;
 $extension =~ s/^\.// ;
 
-my $type =  $extension ne q{}
-			? $extension
-			: 'internal_asciio_format';
+my $type = $extension ne q{} ? $extension : 'internal_asciio_format';
 
 my $title ;
 
@@ -44,36 +50,67 @@ if
 			$self,
 			$file_name,
 			) ;
-		
-	$self->load_self($saved_self) ; # resurect from momified
+	
+	$self->load_self($saved_self) ; # resurrect from mummified
 	$self->{IMPORT_EXPORT_HANDLERS}{HANDLER_DATA} = $handler_data ;
 	}
 else
 	{
-	my $serialized_self = decompress(read_file($file_name)) ;
-	
-	my $VAR1 ;
-	my $saved_self  = eval $serialized_self or die "load_file: can't load file '$file_name': $! $@\n" ;
-	$self->load_self($saved_self) ; # resurect
-	delete $self->{IMPORT_EXPORT_HANDLERS}{HANDLER_DATA} ;
-	
-	$title = $file_name ;
-	}	
+	if(-e $file_name && -s $file_name)
+		{
+		my $serialized_self = decompress(read_file($file_name)) ;
+		my $decoder = get_sereal_decoder() ;
+		my $saved_self = $serialized_self = $decoder->decode($serialized_self) ;
+		
+		if($@)
+			{
+			write_file("failed_resurection_source.pl", {binmode => ':utf8'}, $serialized_self) ;
+			die "load_file: can't load file '$file_name': $! $@\n" ;
+			}
+		
+		$self->load_self($saved_self) ; # resurrect
+		delete $self->{IMPORT_EXPORT_HANDLERS}{HANDLER_DATA} ;
+		delete $self->{CACHE} ;
+		
+		$title = $file_name ;
+		}
+	else
+		{
+		my $element = $self->add_new_element_named('Asciio/box', 0, 0) ;
+		my $box_type = $element->get_box_type() ;
+		$box_type->[1][0] = 1 ; # title separator
+		$element->set_box_type($box_type) ;
+		
+		$element->set_text('Warning!', "'$file_name' has no content.");
+		
+		$self->select_elements(1, $element) ;
+		$self->update_display() ;
+		
+		$title = $file_name ;
+		}
+	}
 
 return $title ;
 }
 
 #-----------------------------------------------------------------------------
 
- # gtk elements memory is handled by Gtk2 module
-Readonly my  @GTK_ELEMENTS => 
+Readonly my  @ELEMENTS_TO_KEEP_AWAY_FROM_CURRENT_OBJECT => 
 	qw
 		(
-		widget PIXMAP 
-		ALLOCATED_COLORS 
+		widget
+		root_window
+		sc_window
 		ACTIONS CURRENT_ACTIONS ACTIONS_BY_NAME
 		HOOKS IMPORT_EXPORT_HANDLERS
 		TITLE
+		ELEMENT_TYPES_BY_NAME
+		ELEMENT_TYPES
+		MIDDLE_BUTTON_SELECTION_FILTER
+		CACHE
+		COLORS
+		ACTION_VERBOSE
+		DO_STACK_POINTER DO_STACK
 		) ;
 
 sub load_self
@@ -82,7 +119,8 @@ my ($self, $new_self)  = @_;
 
 return unless defined $new_self ;
 
-delete @{$new_self}{@GTK_ELEMENTS} ;
+delete @{$new_self}{@ELEMENTS_TO_KEEP_AWAY_FROM_CURRENT_OBJECT} ;
+
 my @keys = keys %{$new_self} ;
 @{$self}{@keys} = @{$new_self}{@keys} ;
 }
@@ -116,8 +154,6 @@ for my $new_element (@{$elements})
 	$new_element->{NAME} =~ s~/+~/~g ;
 	$new_element->{NAME} =~ s~^/~~g ;
 	
-	#~ print $new_element->{NAME} . "\n" ;
-	
 	if(exists $new_element->{NAME})
 		{
 		if(exists $self->{ELEMENT_TYPES_BY_NAME}{$new_element->{NAME}})
@@ -148,12 +184,12 @@ sub save_stencil
 {
 my ($self) = @_ ;
 
-my $name = $self->display_edit_dialog('stencil name') ;
+my $name = $self->display_edit_dialog('stencil name', '', $self) ;
 
 if(defined $name && $name ne q[])
 	{
 	my $file_name = $self->get_file_name('save') ;
-
+	
 	if(defined $file_name && $file_name ne q[])
 		{
 		if(-e $file_name)
@@ -167,19 +203,24 @@ if(defined $name && $name ne q[])
 			$file_name = undef unless $override eq 'yes' ;
 			}
 		}
-
+	
 	if(defined $file_name && $file_name ne q[])
 		{
 		use Data::Dumper ;
 		my ($element) = $self->get_selected_elements(1) ;
 		
+		my $cache = $element->{CACHE} ;
+		delete $element->{CACHE} ;
+		
 		my $stencil = Clone::clone($element) ;
 		
+		$element->{CACHE} = $cache ;
+		
 		delete $stencil->{X} ;
 		delete $stencil->{Y} ;
 		$stencil->{NAME} = $name;
 		
-		write_file($file_name, Dumper [$stencil]) ;
+		write_file($file_name, {binmode => ':utf8'}, Dumper [$stencil]) ;
 		}
 	}
 }
@@ -191,23 +232,36 @@ sub serialize_self
 my ($self, $indent) = @_ ;
 
 local $self->{widget} = undef ;
-local $self->{PIXMAP} = undef ;
-local $self->{ALLOCATED_COLORS} = undef ;
 local $self->{ACTIONS} = [] ;
 local $self->{HOOKS} = [] ;
 local $self->{CURRENT_ACTIONS} = [] ;
 local $self->{ACTIONS_BY_NAME} = [] ;
 local $self->{DO_STACK} = undef ;
+local $self->{DO_STACK_POINTER} = undef ;
 local $self->{IMPORT_EXPORT_HANDLERS} = undef ;
 local $self->{MODIFIED} => 0 ;
 local $self->{TITLE} = '' ;
 local $self->{CREATE_BACKUP} = undef ;
+local $self->{MIDDLE_BUTTON_SELECTION_FILTER} = undef ;
+local $self->{ELEMENT_TYPES} = undef ;
+local $self->{ELEMENT_TYPES_BY_NAME} = undef ;
+local $self->{ACTION_VERBOSE} = undef ;
 
-local $Data::Dumper::Purity = 1 ;
-local $Data::Dumper::Indent = $indent || 0 ;
-local $Data::Dumper::Sortkeys = 1 ;
+my @elements_cache ;
+for my $element (@{$self->{ELEMENTS}}) 
+	{
+	push @elements_cache, [$element, $element->{CACHE}] ;
+	$element->{CACHE} = undef ;
+	}
 
-Dumper($self) ;
+$self->{CACHE}{ENCODER} = my $encoder = $self->{CACHE}{ENCODER} // get_sereal_encoder({compress => SRL_ZLIB}) ;
+local $self->{CACHE} = undef ;
+
+my $serialized = $encoder->encode($self) ;
+
+$_->[0]{CACHE} = $_->[1] for @elements_cache ;
+
+return $serialized ;
 }
 
 #-----------------------------------------------------------------------------
@@ -231,23 +285,22 @@ if
 			$file_name,
 			$self->{IMPORT_EXPORT_HANDLERS}{HANDLER_DATA},
 			) ;
-	
 	}
 else
 	{
 	if($self->{CREATE_BACKUP} && -e $file_name)
 		{
 		use File::Copy;
-		copy($file_name,"$file_name.bak") or die "save_with_type: Copy failed while making backup copy: $!";		
+		copy($file_name,"$file_name.bak") or die "save_with_type: Copy failed while making backup copy: $!" ;
 		}
 		
-	write_file($file_name,compress($self->serialize_self() .'$VAR1 ;')) ;
 	$title = $file_name ;
-	}	
+	write_file($file_name, compress($self->serialize_self() .'$VAR1 ;')) or $title = undef ;
+	}
 	
 return $title ;
 }
 
 #-----------------------------------------------------------------------------
 
-1 ;
\ No newline at end of file
+1 ;
