@@ -1,6 +1,5 @@
 
 package App::Asciio::stripes::editable_arrow2 ;
-
 use base App::Asciio::stripes::stripes ;
 
 use strict;
@@ -14,14 +13,14 @@ use Clone ;
 
 Readonly my $DEFAULT_ARROW_TYPE => 
 	[
-	['Up', '|', '|', '^', 1, ],
-	['45', '/', '/', '^', 1, ],
-	['Right', '-', '-', '>', 1, ],
-	['135', '\\', '\\', 'v', 1, ],
-	['Down', '|', '|', 'v', 1, ],
-	['225', '/', '/', 'v', 1, ],
-	['Left', '-', '-', '<', 1, ],
-	['315', '\\', '\\', '^', 1, ],
+	['Up',    '|',  '|', '^', 1],
+	['45',    '/',  '/', '^', 1],
+	['Right', '-',  '-', '>', 1],
+	['135',  '\\', '\\', 'v', 1],
+	['Down',  '|',  '|', 'v', 1],
+	['225',   '/',  '/', 'v', 1],
+	['Left',  '-',  '-', '<', 1],
+	['315',  '\\', '\\', '^', 1],
 	] ;
 
 sub new
@@ -29,11 +28,11 @@ sub new
 my ($class, $element_definition) = @_ ;
 
 my $self = bless  {}, __PACKAGE__ ;
-	
+
 $self->setup
 	(
 	$element_definition->{ARROW_TYPE} || Clone::clone($DEFAULT_ARROW_TYPE),
-	$element_definition->{END_X}, 	$element_definition->{END_Y},
+	$element_definition->{END_X}, $element_definition->{END_Y},
 	$element_definition->{EDITABLE},
 	) ;
 
@@ -48,13 +47,41 @@ my ($self, $arrow_type, $end_x, $end_y, $editable) = @_ ;
 
 my ($stripes, $real_end_x, $real_end_y) = get_arrow($arrow_type, $end_x, $end_y) ;
 
+my ($ex1, $ey1, $ex2, $ey2) ;
+
+if ($real_end_x < 0)
+	{
+	$ex1 = $real_end_x ;
+	$ex2 = 1 ;
+	}
+else
+	{
+	$ex1 = 0 ;
+	$ex2 = $real_end_x + 1 ;
+	}
+
+if ($real_end_y < 0)
+	{
+	$ey1 = $real_end_y ;
+	$ey2 = 1 ;
+	}
+else
+	{
+	$ey1 = 0 ;
+	$ey2 = $real_end_y + 1 ;
+	}
+
 $self->set
-	(
-	STRIPES => $stripes,
-	END_X => $real_end_x,
-	END_Y => $real_end_y,
+(
+	STRIPES    => $stripes,
+	END_X      => $real_end_x,
+	END_Y      => $real_end_y,
 	ARROW_TYPE => $arrow_type,
-	) ;
+	CACHE      => undef,
+	EXTENTS    => [$ex1, $ey1, $ex2, $ey2],
+	WIDTH      => $ex2 - $ex1,
+	HEIGHT     => $ey2 - $ey1,
+) ;
 }
 
 #-----------------------------------------------------------------------------
@@ -115,7 +142,7 @@ for ($direction)
 		$arrow = $height == 2
 				? $end . "\n" . $start
 				: $end . "\n" . ("$body\n" x ($height -2)) . $start ;
-				
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => $height,
@@ -123,11 +150,11 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => 0,
 			'Y_OFFSET' => $end_y,
-			};
-			
+			} ;
+		
 		last ;
 		} ;
-		
+	
 	$_ eq '45' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[1]}[1 .. 3] ;
@@ -145,8 +172,8 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => 0,
 			'Y_OFFSET' => 0,
-			};
-			
+			} ;
+		
 		for(my $position = -$end_y - 1 ; $position > 0 ; $position--)
 			{
 			push @{$stripes},
@@ -156,9 +183,9 @@ for ($direction)
 				'WIDTH' => 1,
 				'X_OFFSET' => $position,
 				'Y_OFFSET' => -$position,
-				};
+				} ;
 			}
-			
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => 1,
@@ -166,11 +193,11 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => -$end_y ,
 			'Y_OFFSET' => $end_y ,
-			};
+			} ;
 			
 		last ;
 		} ;
-		
+	
 	$_ eq 'right' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[2]}[1 .. 3] ;
@@ -184,7 +211,7 @@ for ($direction)
 				: $width == 2
 					? $start . $end
 					: $start . ($body x ($width -2)) . $end ;
-					
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => 1,
@@ -192,11 +219,11 @@ for ($direction)
 			'WIDTH' => $width,
 			'X_OFFSET' => 0,
 			'Y_OFFSET' => 0,
-			};
+			} ;
 			
 		last ;
 		} ;
-		
+	
 	$_ eq '135' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[3]}[1 .. 3] ;
@@ -214,8 +241,8 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => 0 ,
 			'Y_OFFSET' => 0 ,
-			};
-			
+			} ;
+		
 		for(my $position = 1 ; $position < $end_y ; $position++)
 			{
 			push @{$stripes},
@@ -225,9 +252,9 @@ for ($direction)
 				'WIDTH' => 1,
 				'X_OFFSET' => $position,
 				'Y_OFFSET' => $position,
-				};
+				} ;
 			}
-			
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => 1,
@@ -235,11 +262,11 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => $end_y ,
 			'Y_OFFSET' => $end_y ,
-			};
+			} ;
 			
 		last ;
 		} ;
-		
+	
 	$_ eq 'down' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[4]}[1 .. 3] ;
@@ -250,7 +277,7 @@ for ($direction)
 		$arrow = $height == 2
 				? $start . "\n" . $end
 				: $start . "\n" . ("$body\n" x ($height -2)) . $end ;
-				
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => $height,
@@ -258,10 +285,11 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => 0,
 			'Y_OFFSET' => 0,
-			};
+			} ;
+		
 		last ;
 		} ;
-		
+	
 	$_ eq '225' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[5]}[1 .. 3] ;
@@ -279,8 +307,8 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => 0,
 			'Y_OFFSET' => 0,
-			};
-			
+			} ;
+		
 		for(my $position = $end_y - 1 ; $position > 0 ; $position--)
 			{
 			push @{$stripes},
@@ -290,9 +318,9 @@ for ($direction)
 				'WIDTH' => 1,
 				'X_OFFSET' => -$position,
 				'Y_OFFSET' => $position,
-				};
+				} ;
 			}
-			
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => 1,
@@ -300,11 +328,11 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => -$end_y ,
 			'Y_OFFSET' => $end_y ,
-			};
-			
+			} ;
+		
 		last ;
 		} ;
-		
+	
 	$_ eq 'left' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[6]}[1 .. 3] ;
@@ -317,7 +345,7 @@ for ($direction)
 		$arrow = $width == 2
 				? $end . $start
 				: $end . ($body x ($width -2)) . $start ;
-				
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => 1,
@@ -325,11 +353,11 @@ for ($direction)
 			'WIDTH' => $width,
 			'X_OFFSET' => $end_x,
 			'Y_OFFSET' => 0,
-			};
-			
+			} ;
+		
 		last ;
 		} ;
-		
+	
 	$_ eq '315' and do
 		{
 		my ($start, $body, $end) = @{$arrow_type->[7]}[1 .. 3] ;
@@ -347,8 +375,8 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => 0,
 			'Y_OFFSET' => 0,
-			};
-			
+			} ;
+		
 		for(my $position = 1 ; $position < -$end_y ; $position++)
 			{
 			push @{$stripes},
@@ -358,9 +386,9 @@ for ($direction)
 				'WIDTH' => 1,
 				'X_OFFSET' => -$position,
 				'Y_OFFSET' => -$position,
-				};
+				} ;
 			}
-			
+		
 		push @{$stripes},
 			{
 			'HEIGHT' => 1,
@@ -368,8 +396,8 @@ for ($direction)
 			'WIDTH' => 1,
 			'X_OFFSET' => $end_y,
 			'Y_OFFSET' => $end_y,
-			};
-			
+			} ;
+		
 		last ;
 		} ;
 	}
@@ -383,10 +411,7 @@ sub get_extra_points
 {
 my ($self) = @_ ;
 
-return
-	(
-	{X =>  $self->{END_X}, Y => $self->{END_Y}, NAME => 'resize'},
-	) ;
+return ( {X =>  $self->{END_X}, Y => $self->{END_Y}, NAME => 'resize'},) ;
 }
 
 #-----------------------------------------------------------------------------
@@ -421,155 +446,23 @@ return(0, 0, $self->{END_X} + 1, $self->{END_X} + 1) ;
 
 #-----------------------------------------------------------------------------
 
-sub get_text
-{
-my ($self) = @_ ;
-}
+sub get_text { my ($self) = @_ ; }
 
 #-----------------------------------------------------------------------------
 
-sub set_text
-{
-my ($self) = @_ ;
-}
+sub set_text { my ($self) = @_ ; }
 
 #-----------------------------------------------------------------------------
 
 sub edit
 {
-my ($self) = @_ ;
+my ($self, $asciio) = @_ ;
 
-return unless $self->{EDITABLE} ;
-
-display_box_edit_dialog($self->{ARROW_TYPE}) ;
+$self->display_box_edit_dialog() ;
 
 $self->setup($self->{ARROW_TYPE}, $self->{END_X}, $self->{END_Y}, $self->{EDITABLE}) ;
 }
 
-use Glib ':constants';
-use Gtk2 -init;
-use Glib qw(TRUE FALSE);
-
-sub display_box_edit_dialog
-{
-my ($rows) = @_ ;
-
-my $window = new Gtk2::Window() ;
-
-my $dialog = Gtk2::Dialog->new('Arrow attributes', $window, 'destroy-with-parent')  ;
-$dialog->set_default_size (220, 270);
-$dialog->add_button ('gtk-ok' => 'ok');
-
-#~ my $vbox = $dialog->vbox ;
-my $dialog_vbox = $dialog->vbox ;
-
-my $vbox = Gtk2::VBox->new (FALSE, 5);
-$dialog_vbox->pack_start ($vbox, TRUE, TRUE, 0);
-
-$vbox->pack_start (Gtk2::Label->new (""),
-		 FALSE, FALSE, 0);
-
-my $sw = Gtk2::ScrolledWindow->new;
-$sw->set_shadow_type ('etched-in');
-$sw->set_policy ('automatic', 'automatic');
-$vbox->pack_start ($sw, TRUE, TRUE, 0);
-
-# create model
-my $model = create_model ($rows);
-
-# create tree view
-my $treeview = Gtk2::TreeView->new_with_model ($model);
-$treeview->set_rules_hint (TRUE);
-$treeview->get_selection->set_mode ('single');
-
-add_columns($treeview, $rows);
-
-$sw->add($treeview);
-
-$treeview->show() ;
-$vbox->show() ;
-$sw->show() ;
-
-$dialog->run() ;
-
-$dialog->destroy ;
-}
-
-#-----------------------------------------------------------------------------
-
-sub create_model 
-{
-my ($rows) = @_ ;
-
-my $model = Gtk2::ListStore->new(qw/Glib::String Glib::String  Glib::String  Glib::String Glib::Boolean/);
-
-foreach my $row (@{$rows}) 
-	{
-	my $iter = $model->append;
-
-	my $column = 0 ;
-	$model->set ($iter, map {$column++, $_} @{$row}) ;
-	}
-
-return $model;
-}
-
-#-----------------------------------------------------------------------------
-
-sub add_columns 
-{
-my ($treeview, $rows) = @_ ;
-my $model = $treeview->get_model;
-
-# column for row titles
-my $row_renderer = Gtk2::CellRendererText->new;
-$row_renderer->set_data (column => 0);
-
-$treeview->insert_column_with_attributes
-			(
-			-1, '', $row_renderer,
-			text => 0,
-			) ;
-my $column = $treeview->get_column(0) ;
-$column->set_sizing('fixed') ;
-$column->set_fixed_width(80) ;
-
-my $current_column = 1 ;
-for my $column_title('start', 'body', 'end')
-	{
-	my $renderer = Gtk2::CellRendererText->new;
-	$renderer->signal_connect (edited => \&cell_edited, [$model, $rows]);
-	$renderer->set_data (column => $current_column );
-
-	$treeview->insert_column_with_attributes 
-				(
-				-1, $column_title, $renderer,
-				text => $current_column,
-				editable => 4, 
-				);
-				
-	$current_column++ ;
-	}
-}
-
-#-----------------------------------------------------------------------------
-
-sub cell_edited 
-{
-my ($cell, $path_string, $new_text, $model_and_rows) = @_;
-
-my ($model, $rows) = @{$model_and_rows} ;
-
-my $path = Gtk2::TreePath->new_from_string ($path_string);
-my $column = $cell->get_data ("column");
-my $iter = $model->get_iter($path);
-my $row = ($path->get_indices)[0];
-
-$rows->[$row][$column] = $new_text ;
-
-$model->set($iter, $column, $new_text);
-}
-
 #-----------------------------------------------------------------------------
 
 1 ;
