@@ -5,7 +5,7 @@
 
 register_hooks
 	(
-	['CANONIZE_CONNECTIONS' => \&canonize_connections],
+	{ CANONIZE_CONNECTIONS => \&canonize_connections},
 	) ;
 
 #----------------------------------------------------------------------------------------------
@@ -41,8 +41,12 @@ for my $connection (@{$connections})
 	{
 	if
 		(
-		ref $connection->{CONNECTED} eq 'App::Asciio::stripes::section_wirl_arrow'
+			(
+			ref $connection->{CONNECTED} eq 'App::Asciio::stripes::section_wirl_arrow' ||
+			ref $connection->{CONNECTED} eq 'App::Asciio::stripes::angled_arrow'
+			)
 		&& $connection->{CONNECTED}->is_autoconnect_enabled()
+		&& $connection->{CONNECTEE}->is_autoconnect_enabled()
 		)
 		{
 		reconnect_section_wirl_arrow($connection)  ;
@@ -62,9 +66,12 @@ my ($start_name, $end_name) = ($connectors[0]{NAME}, $connectors[-1]{NAME}) ;
 if($connection->{CONNECTOR}{NAME} eq $end_name)
 	{
 	# end connector
+	
+	return unless $connectee->is_optimize_enabled() ;
+	
 	my ($connectee_x, $connectee_y, $connectee_width, $connectee_hight) = 
 		($connectee->{X}, $connectee->{Y}, $connectee->get_size()) ;
-
+	
 	my $connected_x = $connected->{X} + $connectors[-2]{X};
 	my $connected_y = $connected->{Y} + $connectors[-2]{Y};
 	
@@ -140,12 +147,14 @@ if($connection->{CONNECTOR}{NAME} eq $end_name)
 else
 	{
 	# start connector
+	return unless $connectee->is_optimize_enabled() ;
+	
 	my ($connectee_x, $connectee_y, $connectee_width, $connectee_hight) = 
 		($connectee->{X}, $connectee->{Y}, $connectee->get_size()) ;
-
+	
 	my $end_connector_x = $connected->{X} + $connectors[1]{X};
 	my $end_connector_y = $connected->{Y} + $connectors[1]{Y} ;
-
+	
 	if($end_connector_x < $connectee_x)
 		{
 		# arrow ends on left of the box
@@ -240,5 +249,3 @@ if($asciio_connection->{CONNECTION}{NAME} ne $connection_name)
 	}
 }
 
-	
-	
\ No newline at end of file
